#!/bin/sh
# BuildAll: use this to build texdist and its required frameworks from the command line
# XCode 3 and 4 compliant, version 1
echo This command will build the TeXDist preference pane
pushd "$(dirname "$0")"
echo Finding Xcode 3
tempfoo="$(basename $0)"
TMPDIR="$(mktemp -d /tmp/${tempfoo}.XXXXXX)" || exit 1
echo The temporary directory is $TMPDIR
mkdir -p "$TMPDIR"
ls /*/Applications/Xcode.app/Contents/Info.plist|while read file
do
	plutil -convert xml1 -o "$TMPDIR/Info.plist" "$file"
	version="$(perl -e 'local $/="";while(<>){print "$1" if m/CFBundleShortVersionString.*?<string>(\d)\..*/s ;}' "$TMPDIR/Info.plist")"
	echo "$(dirname $(dirname $(dirname $(dirname "$file"))))" > "$TMPDIR/Xcode${version}"
done
if [ -f "$TMPDIR/Xcode3" ]
then
	if [ -z "$(xcodebuild -version|grep 'Xcode 3')" ]
	then
#		we do not use Xcode3 whereas we should
		OLD_PATH="$(xcode-select -print-path)"
		echo sudo xcode-select -switch "$(cat "$TMPDIR/Xcode3")"
		sudo xcode-select -switch "$(cat "$TMPDIR/Xcode3")"
		rm -Rf "$TMPDIR"
		if [ -z "$(xcodebuild -version|grep 'Xcode 3')" ]
		then
			echo "Bad xcode version";
			exit -1;
		fi
	fi
fi
echo The selected xcode is with $(xcode-select -print-path)
cd "$(dirname "$0")"
action="$1"
if [ -z "$action" ]
then
	action="build"
fi
echo xcodebuild -project TeXDist.xcodeproj -target "Package the products" clean build
xcodebuild -project TeXDist.xcodeproj -target "Package the products" clean build
if ! [ -z "$OLD_PATH" ]
then
	echo sudo xcode-select -switch "$OLD_PATH"
	sudo xcode-select -switch "$OLD_PATH"
fi

suffix=""
executables=(GetSourceVersion TeXDistUtility TeXDistPrefPane TeXDistTool)
architectures=(ppc i386 x86_64)
answers=("flags 0x0 RR" "flags 0x2 GC RR" "flags 0x2 OBJC_IMAGE_SUPPORTS_GC")
echo Testing architectures and garbage collection in TeXDistPrefPane$suffix.prefPane
for i in 0 1 2 3
do
	executable="${executables[$i]}"
	for j in 0 1 2
	do
		architecture="${architectures[$j]}"
		answer="${answers[$j]}"
		if [ -z "$(otool -arch "$architecture" -oV DerivedData/TeXDist/Build/Products/TeXDistPrefPane$suffix.prefPane/Contents/MacOS/$executable | grep "$answer")" ]
		then
			echo "FAILURE: $executable($architecture) bad garbage collection management"
		else
			echo "OK: $executable($architecture)"
		fi
	done
done
exit 0
